import java.io.*;
import java.util.*;


public class DWTree extends M865Tree {

	public DWTree(M865Tree tree) {
		super(tree);
	}
	
	public DWTree(M865Graph graph) {
		super(graph);
	}

	public DWTree(SortedSet vertexSet, SortedSet edgeList) {
		super(vertexSet, edgeList);
	}
		
	public DWTree() {
		super();
	}

	public Object clone() {
		return new DWTree((SortedSet)((TreeSet)vertexSet).clone(), (SortedSet)((TreeSet)edgeList).clone());
	}

	protected void buildCertificate(DWTree t) {
		// Make a temporary copy of the tree which we can manipulate.
		DWTree tree = (DWTree)t.clone();

		// Initialize the labels for each vertex
		TreeMap labelOf = new TreeMap();
		Iterator iter = tree.getVertexSet().iterator();
		while (iter.hasNext()) {
			labelOf.put(iter.next(),"01");
		}

		// Make multiple passes over the tree, stripping leaves and building the labels
		M865PriorityQueue pq = new DWhvPriorityQueue(tree.getVertexSet().size());
		Vector leaves = new Vector();
		int pass = 0;
		String s, pad = "             ";
		while (2 < tree.getVertexSet().size()) {
			// System.out.println("\nPass " + pass++);
			iter = tree.getVertexSet().iterator();

			// Build a new label for every interior vertex 
			while (iter.hasNext()) {
				M865Vertex v = (M865Vertex)iter.next();
				// s = v.toString() + ":";
				// System.out.println(s + pad.substring(s.length()) + (String)labelOf.get(v));
				if (1 < tree.degreeOf(v)) {
					String label = (String)labelOf.get(v);
					pq.enqueue(label.substring(1,(label.length()-1)));
					SortedSet neighbors = tree.neighborhoodOf(v);
					Iterator k = neighbors.iterator();
					while (k.hasNext()) {
						M865Vertex leaf = (M865Vertex)k.next();
						if (tree.degreeOf(leaf) == 1) {
							pq.enqueue((String)labelOf.get(leaf));
							leaves.add(leaf); 
						}
					}
					// Build the label
					StringBuffer newLabel = new StringBuffer("0");
					while (!pq.isEmpty()) {
						newLabel.append((String)pq.dequeue());
					}
					newLabel.append("1");
					labelOf.put(v,newLabel.toString());
				}
			}

			// Remove all the leaves that were found on this pass
			iter = leaves.iterator();
			while (iter.hasNext()) {
				tree.removeVertex((M865Vertex)iter.next());
			}
			leaves.clear();
		}
		center = (M865Vertex)tree.getVertexSet().first();
		String label1 = (String)labelOf.get(center);
		String label2;
		if (tree.getVertexSet().size() == 2) {
			label2 = (String)labelOf.get(tree.getVertexSet().last());
			if (0 < label1.compareTo(label2)) {
				certificate = label2 + label1;
			} else {
				certificate = label1 + label2;
			}
		} else {
			certificate = label1;
		}
		updateCertificate = false;
	}	


	protected void buildRootedSubTree(M865Vertex p) {
		M865TreeNode pNode = (M865TreeNode)nodeOf.get(p);
		M865Vertex g = pNode.getParent().getVertex();
		SortedSet neighbors = neighborhoodOf(p);
		Iterator iter = neighbors.iterator();
		while (iter.hasNext()) {
			M865Vertex v = (M865Vertex)iter.next();
			if (!g.equals(v)) {
				pNode.addChild((M865TreeNode)nodeOf.get(v));
				buildRootedSubTree(v);
			}
		}
	}
	
	public M865TreeNode buildRootedTree(M865Vertex rootVertex) {
		if (!vertexSet.contains(rootVertex)) {
			throw (new UnsupportedOperationException("Vertex is not in this tree"));
		}
		nodeOf = new TreeMap();
		Iterator iter = vertexSet.iterator();
		while (iter.hasNext()){
			M865Vertex v = (M865Vertex)iter.next();
			nodeOf.put(v, new M865TreeNode(v, null, null, null));
		}
		if (updateNeighborhoods) {
			buildNeighborhoods();
		}
		root = (M865TreeNode)nodeOf.get(rootVertex);
		SortedSet neighbors = neighborhoodOf(rootVertex);
		iter = neighbors.iterator();
		while (iter.hasNext()) {
			M865Vertex v = (M865Vertex)iter.next();
			root.addChild((M865TreeNode)nodeOf.get(v));
			buildRootedSubTree(v);
		}
		updateRootedTree = false;
		return root;
	}

	public String getCertificate() {
		if (updateCertificate) {
			buildCertificate(this);
		}
		return certificate;
	}

	public M865Vertex getCenter() {
		if (updateCertificate) {
			buildCertificate(this);
		}
		return center;
	}

	public void printTree() {
		if (updateRootedTree) {
			M865Vertex rootVertex = (M865Vertex)vertexSet.first();
			buildRootedTree(rootVertex);
		}
		root.printTree();
	}
	
	public static void main (String args[])
		throws IOException {

		M865GraphReader graphReader;
		if (0 < args.length) {
			graphReader = new M865GraphReader(args[0]);
		} else {
			graphReader = new M865GraphReader();
		}
		graphReader.readEdgeList();
		graphReader.dump();
		
		M865Graph g = graphReader.getGraph();
		System.out.println(g);
		System.out.println("nC: " + g.getNumberOfComponents());
		System.out.println("Connected: " + g.isConnected());
		System.out.println("Has cycles: " + g.hasCycles());
		SortedSet vSet = g.getVertexSet();
		if (g.isConnected() && !g.hasCycles()) {
			System.out.println("This is a tree!");
			DWTree tree = new DWTree(g);
			System.out.println(tree);
			tree.buildRootedTree((M865Vertex)vSet.first());
			tree.printTree();
			System.out.println("Certificate: " + tree.getCertificate());
			tree.buildRootedTree(tree.getCenter());
			tree.printTree();
		} else {
			System.out.println("This is NOT a tree!");
		}			
		TreeMap tm = new TreeMap();
		Iterator iter = vSet.iterator();
		while (iter.hasNext()) {
			M865Vertex vertex = (M865Vertex)iter.next();
			System.out.println("Degree of "+vertex+" is " + g.degreeOf(vertex));
		}
	}
}